import com.amazonaws.services.s3.AmazonS3ClientBuilder;
import com.amazonaws.services.s3.transfer.TransferManagerBuilder;
import lombok.RequiredArgsConstructor;
import lombok.SneakyThrows;
import org.apache.commons.lang3.time.StopWatch;

import java.io.File;
import java.io.FileWriter;

public class TestDirCopy {
    public static void main(String[] args) throws InterruptedException {
        var s3 = AmazonS3ClientBuilder.standard().build();
        var transferManager = TransferManagerBuilder.standard().withS3Client(s3).build();

        Thread.ofVirtual().start(new SlowDirectoryWriter());
        Thread.sleep(100);

        var directory = new File("c:/testDir");

        var multipleFileUpload = transferManager.uploadDirectory("ai-ocr-ocr-processing-image", "dirCopy_2", directory, true);
        var stopWatch = StopWatch.createStarted();
        while (multipleFileUpload.getProgress().getPercentTransferred() < 100) {
            if (5000 < stopWatch.getTime()) {
                throw new RuntimeException("timeout");
            }
            System.out.println("progress:" + multipleFileUpload.getProgress().getPercentTransferred());
            var subCount = 1;
            for (var sub : multipleFileUpload.getSubTransfers()) {
                System.out.println(subCount + " / " + sub.getState() + " / " + sub.getProgress());
                subCount++;
            }
            Thread.sleep(1000);
        }
        System.out.println("end");
    }

    static class SlowDirectoryWriter implements Runnable {
        @SneakyThrows
        @Override
        public void run() {
            for (var fileNumber = 1; fileNumber <= 10; fileNumber++) {
                Thread.ofVirtual().start(new SlowFileWriter(fileNumber));
                Thread.sleep(10);
            }
        }
    }

    @RequiredArgsConstructor
    static class SlowFileWriter implements Runnable {
        final int fileNumber;

        @SneakyThrows
        @Override
        public void run() {
            var file = new File("c:/testDir/" + fileNumber + ".txt");
            var fileWriter = new FileWriter(file);
            for (var line = 1; line <= 10 * 1000; line++) {
                fileWriter.write("line: %5d\r\n".formatted(line));
                fileWriter.flush();
            }
            fileWriter.close();
        }
    }
}
